# This script attach texture from image to face of object
# Additional modules needed: numpy, scipy
import Blender
import numpy
import scipy
from Blender import Mesh,Types
from rahat.calibim import *
from rahat.bfunc import *

# CONSTANTS
MINVISPIX = 1 # the minimum of visible pixels
MINVISIBL = 0.50 # the minimum of visible/hidden pixels of the face
MINANGLE = 90 # the minimum angle between camera axis and the face normal
DEBUG = True
def attachTex( meshObject, listCI):
    """
    def attachTex( meshObject, listCI): convert images to texures,
    attaches it to a object from the best view
    listCI is the list of CalibIm objects
    meshObject is the blender object with meshdata
    """
    if DEBUG:
        print "attachTex( ",meshObject,",", listCI,")"
    # first we get visibility of all faces in each image
    materials = list()  # list of materials of the object
    uvImages = list()   # the list of uv coordinates in each image
    pixImages = list()  # the list of visible, hidden pixels of each face in each image
    meshData=meshObject.getData(False,True) # get the mesh data
    meshData.faceUV = 1
    visibleFaces = list()   # the list of visible faces
    hiddenFaces = list()    # the list of hidden faces
    # for each image, we make a material,texture
    # render into it and get the uv coordinates for vertices
    # and get the visible and hidden pixels
    for ci in listCI:
        ci.toTexture()  # create a texture
        # this doesnt work
        #meshObject.getData().materials.append(ci.getMaterial())
        materials.append( ci.getMaterial() )    # append it to materials list
        uvImages.append( ci.getFaceUVs( meshObject)) # get the uv coords
        pixImages.append( ci.getFacePixels( meshObject)) # get the vis,hid pixels
    # set the list of materials of this object
    # not sure how is this working, so using both functions
    print "materials list:",materials
    #meshObject.setMaterials( materials)
    #meshObject.getData().setMaterials( materials) this doesnt work
    
    meshObject.getData(False,True).materials = materials
    meshObject.colbits=0x00 # this should say, that we use only mesh materials
    meshObject.getData().hasFaceUV( 1) # turn on UV textures
    meshObject.getData().update()
    # get the final mesh representation modified by armatures, parents etc.
    defMesh = getDeformedMeshData( meshObject)
    # objects rotation matrix
    objRot = meshObject.getMatrix().rotationPart()
    # now we will proces each face of the object    
    faceId = 0
    for face in meshData.faces:
        bestVisibility = 0 # the best visible/hidden pixels
        bestVisImgId = -1 # the best visibility image Id
        imgId = 0
        for img in pixImages:
            visible = 0
            hidden = 1
            # looking for the record for this face
            for pixs in img:
                if pixs[0] == faceId:
                    visible = pixs[1]
                    hidden = pixs[2]
                    #print "face: ",faceId," visible: ",visible,", hidden: ",hidden
            # computing visibility
            Visibility = 0
            if hidden != 0:
                Visibility = visible/hidden
            else:
                Visibility = visible
            if (visible > MINVISPIX) and (Visibility > bestVisibility):
                bestVisibility = Visibility
                bestVisImgId = imgId
            imgId = imgId+1
        # after processing all the images, attach the texture
        if (bestVisImgId != -1) and (bestVisibility > MINVISIBL):
            UVs = uvImages[ bestVisImgId]
            uv = UVs[ faceId]
            # set the uv coords
            #print "Setting face: ",faceId," type:",type(face)," with image:",listCI[ bestVisImgId].getImage().getName()," uv: ",uv," mat index:",bestVisImgId
            face.image = listCI[ bestVisImgId].getImage()
            face.mat = bestVisImgId
            #face.flag = NMesh.FaceFlags['ACTIVE']
            face.mode = NMesh.FaceModes['TEX']
            #face.sel = 1
            i=0
            #for fuv in uv:
                
                #print "Seting",i," uv:",uv[i]," last state:",fuv
                #face.uv[i] = Vector([fuv[0],fuv[1]])
                #fuv = Vector(uv[i][0],uv[i][1])
                #fuv[1]= uv[i][1]
                #i=i+1
            face.uv = uv
            visibleFaces.append(faceId)
            #print "should be:",uv," and is:",face.uv
            #meshObject.getData().removeFace( face)
            #meshObject.getData().addFace( face)
            meshObject.getData(False,True).update()
        # if there was not found any correspondence for this face
        # we will add it to the list of invisible faces        
        else:
            hiddenFaces.append(faceId)
            # get the corresponding face
            #defFace = defMesh.faces[faceId]
            #bestVisImgId=-1 # best image ID
            #cimIndex=0 # calibimg index
            #bestangle=0 # best angle between camera axis and face normal
            #for cim in listCI:
                # Get the projection matrix and transpose it to OpenGL format
            #    T=Matrix(cim.getPMatrix()).transpose().rotationPart()
            #    finalNormal=Vector(defFace.no)*objRot*T
                #finalNormal=Vector(defFace.no)*T
            #    angle=AngleBetweenVecs(finalNormal,Vector(0,0,1))
            #    if (angle > MINANGLE) & (angle > bestangle):
                #if angle > bestangle:
            #        bestangle=angle
            #        bestVisImgId=cimIndex
            #    cimIndex=cimIndex+1
            #if bestVisImgId != -1:
            #    UVs = uvImages[ bestVisImgId]
            #    uv = UVs[ faceId]
                # set the uv coords                
            #    face.image = listCI[ bestVisImgId].getImage()
            #    face.mat = bestVisImgId
            #    face.mode = NMesh.FaceModes['TEX']
            #    face.uv = uv
            #    meshObject.getData(False,True).update()
        # increment at the end
        faceId = faceId+1
    # now we will process the invisible faces
    # we will attach texture using the neighborhood faces
    loopcount=10; # this is the maximum of the loop runs
    print "invisible : visible faces"
    # FOR TESTING YOU CAN DISABLE THIS PART
    #loopcount=0
    while (len(hiddenFaces)>0) and (loopcount>0):
        print len(hiddenFaces),":",len(visibleFaces)
        newVis=list() # new visible faces list
        # we will go thrugh all the faces
        for faceId in hiddenFaces:
            # get the face
            hidF = meshData.faces[faceId] 
            hidVer = hidF.v # and its vertices
            # and find the neighbourhoods faces of this face
            neighs=list()            
            for faceIdV in visibleFaces:
                visF = meshData.faces[faceIdV]
                visVer = visF.v
                if ((visVer[0] == hidVer[0]) or
                    (visVer[1] == hidVer[0]) or
                    (visVer[2] == hidVer[0]) or
                    (visVer[0] == hidVer[1]) or
                    (visVer[1] == hidVer[1]) or
                    (visVer[2] == hidVer[1]) or
                    (visVer[0] == hidVer[2]) or
                    (visVer[1] == hidVer[2]) or
                    (visVer[2] == hidVer[2])):
                        neighs.append(visF.mat)
                        newVis.append(faceIdV)                                      
            # now we will check the materials of neighbourhoods
            bestM = -1
            bestMcount = 0
            # compute the occurance of each material                
            for m in neighs:
                if bestMcount < neighs.count(m):
                    bestM = m
                    bestMcount = neighs.count(m)
            # if we found the max used material
            if bestM != -1:
                UVs = uvImages[ bestM]
                uv = UVs[ faceId]
                face=hidF
                # set the uv coords
                face.image = listCI[ bestM].getImage()
                face.mat = bestM
                #face.flag = NMesh.FaceFlags['ACTIVE']
                face.mode = NMesh.FaceModes['TEX']
                face.uv = uv
                # and remove it from the invisibles list
                hiddenFaces.remove(faceId)
                newVis.append(faceId)
        # decrease the loop count
        loopcount=loopcount-1
        # and set the new visible faces
        visibleFaces = newVis
    # update data
    meshObject.getData(False,True).update()
# This is for testing purpose only, comment this please
