#
# bfunc.py
#
# 1.8.2005 Mazany Ondrej - email:mazany.o@seznam.cz
#
# This file contains functions for working with Blender.
# It needs a camera, parsecam and parseanim modules.
#
# setUpFrames() - this function works with global variable frames - list of frames objects
#                 it prepares frames defined in animation.xml file. The list is created during
#                 run of the parseanim() function
#
# runRender() - this function starts animation rendering for each camera in cameras list into
#               directories defined by renderpath in animation.xml file and plus the name of camera
#

# including needed modules
from rahat.calibcam import *                   # own module containing the camera class
from rahat.parsecam import *                 # own module for work with cameras, contains cameras list
from rahat.parseanim import *                # own module for work with frames, contains frames list

# importing required modules
import Blender 
from Blender import *
from Blender.Scene import Render


# seting the frames up in blender
def setUpFrames():
    "Setting all defined frames up"
    global frames       # list of frames
    for frm in frames:  # for every frame
        frm.setUp()     # seting up the given frame in Blender

# run rendering for each camera
def runRender(outputpath,cameras):
    "Starts rendering for each camera"    
    scn = Scene.GetCurrent()                # obtain current scene in Blender
    context = scn.getRenderingContext()     # get the scene data 
    for cam in cameras: # for every camera in list        
        cam.setUp()                                                 # set the camera up in Blender and output format
        print "Rendering into:",outputpath + "/" + cam.name + "/"
        context.setRenderPath(outputpath + "/" + cam.name + "/")    # setting path where to render to
        context.renderAnim()                                        # render animation for current camera
 
    
# Makes a copy of the Mesh object with deformed data
def getDeformedMeshData(ob):
    name = ob.getName()  
    num = 0
    new_mesh = Blender.NMesh.GetRawFromObject(name)
    return new_mesh

