#
# animobject.py
#
#
# Definition of animobject class 
#
# 1.8.2005 Mazany Ondrej - email:mazany.o@seznam.cz
#
# This class is used for storing information about objects in scene which will be animated.
# It has three main attributes - name - name of object in Blender scene
#                                Position and Rotation
# The attribs attribute contains list of attributes of object which should be changed, like positon
# in x coord (px) or rotation around z axis (rz)                             
#

# importing required modules
import Blender 
from Blender import Armature
from Blender.Object import *
from Blender.Armature import *
#from Blender.Armature.Bone import *
from Blender import Mathutils
from Blender.Mathutils import *
from math import *

class Bone:
    "A Bone class which is used by animator project for Blender armature configuration"
    def __init__(self):
        "A class initialization"
        self.name = ""                           # name of bone
        self.attribs = list()                    # attributes of this bone to be setted up
        self.Rotation = [ 0.0, 0.0, 0.0]         # rotation of this bone
    def setUp(self,object): # configures bone
        "Sets the bone up"
        try:
            # First we need to find the Armature associated with this object            
            if (object.getType() == 'Armature'): # if the datablock associated with object is armature
                armature = object.getData();
                bones = armature.getBones()    # now we want to find bone in this armature
                #print bones
                bone = None # init
                for b in bones:
                    if b.getName() == self.name:
                        bone = b;
                if bone <> None:    # if the bone was found                    
                    quat = bone.getQuat() # get the bone quaternion (the x,y,z,w values)
                    rot = quat.toEuler() # get the rotations in euler, rot=[0.0,0.0,0.0] # the rotation we will change
                    for at in self.attribs: # check which attributes should be changed
                        if at == "rx":
                            rot.x = degrees(self.Rotation[0])
                        if at == "ry":
                            rot.y = degrees(self.Rotation[1])
                        if at == "rz":
                            rot.z = degrees(self.Rotation[2])
                    try:
                        print rot
                        quat = rot.toQuat() # convert to quaternation
                        print "xx"
                        bone.setQuat(quat)                # sets quaternation of bone x,y,z,w
                        print "uu"
                        action = object.getAction()       # we need action associated with this object
                        print "aa"
                        if action is None:  # if doesn't existst, we create one
                            print action
                            action = Blender.Armature.NLA.NewAction()
                            action.setActive(object) # and set it active
                            bone.setPose([LOC,ROT,SIZE],action)
                        else:
                            bone.setPose([LOC,ROT,SIZE])
                    except:                               # this tell Blender to remember objects position
                        print "Unable to set bones action!"        # and rotation and size in current frame
                    print "Bone",self.name,"configured : rot=",self.Rotation,"."
                else:
                    print "!! The bone",self.name,"was not found in the armature",armature.getName()," !!"
            else: 
                # problem, we do not have a armature object in object property
                print "!! This object",object.getName()," is not an armature !!"
        except:
            print "!! Problem setting",self.name,"bone !!"
