# Script to copy bvh movement to armature

import Blender
import obj_import
import math
from Blender import Armature,Ipo
from Blender.Mathutils import *
from Blender import Object
from Blender.Object import Pose

skelDict={  "sacroiliac":["Hips","Chest"],
            "vl5":["Chest","Neck"],
            "vl5_to_l_sternoclavicular":["Neck","LeftCollar"],
            "l_sternoclavicular":["LeftCollar","LeftShoulder"],
            "l_shoulder":["LeftShoulder","LeftElbow"],
            "l_elbow":["LeftElbow","LeftWrist"],
            "l_wrist":["LeftWrist","LeftWrist_end"],
            "vl5_to_r_sternoclavicular":["Neck","RightCollar"],
            "r_sternoclavicular":["RightCollar","RightShoulder"],
            "r_shoulder":["RightShoulder","RightElbow"],
            "r_elbow":["RightElbow","RightWrist"],
            "r_wrist":["RightWrist","RightWrist_end"],
            "HumanoidRoot_to_l_hip":["Hips","LeftHip"],
            "l_hip":["LeftHip","LeftKnee"],
            "l_knee":["LeftKnee","LeftAnkle"],
            "l_ankle":["LeftAnkle","LeftAnkle_end"],
            "HumanoidRoot_to_r_hip":["Hips","RightHip"],
            "r_hip":["RightHip","RightKnee"],
            "r_knee":["RightKnee","RightAnkle"],
            "r_ankle":["RightAnkle","RightAnkle_end"],
            "vt3":["Head","Head_end"]
            # the reference for the rest of the bones was not found
            # it is item to consider if it is not better to change the 
            # skeleton instead of using h-anim definitions
            }
# the bones that will change relativelly to the rest position
onlyDif=[#"sacroiliac","vl5","vt3",
        "l_ankle","r_ankle",
        #"l_knee","r_knee",
        #"l_hip","r_hip",
        "HumanoidRoot_to_l_hip","HumanoidRoot_to_r_hip"]

onlyDifVecsDict={}

# setting the object names that we will use during the 
# measurement
bvhTop="Head_end"
bvhBottom="RightAnkle_end"
bvhRoot="Hips"

def getQuat(vec1,vec2):
    """
    def getQuat(vec1,vec2): 
        returns a quaternion object which holds the needed rotation
        to transform vec1 to vec2
    """
    v1=Vector(vec1)
    v2=Vector(vec2)
    v1.normalize()
    v2.normalize()
    n=CrossVecs(v1,v2)
    dotp=DotVecs(v1,v2)
    phi=math.acos(float(dotp))
    #if vec3 == Vector(0.0,0.0,0.0):
    #    return Quaternion(1.0,0,0,0)
    #print math.sqrt(2)
    pars=[math.cos(phi/2), math.sin(phi/2)*n[0], math.sin(phi/2)*n[1], math.sin(phi/2)*n[2]]
    #print "pars:",pars
    #qt = Quaternion( n, AngleBetweenVecs(vec1,vec2))    
    qt = Quaternion(math.cos(phi/2), math.sin(phi/2)*n[0], math.sin(phi/2)*n[1], math.sin(phi/2)*n[2])
    #qt = Quaternion(dotp, n[0], n[1], n[2])
    
    #print "qt=",qt
    #transformedVec1=vec1*qt.toMatrix()
    # becouse AngleBetweenVecs returns absolute angle,
    # we need to check the proper transformation
    # it there is not a difference between transformed and final vec
    # (note that due to float numbers precision)
    # transformed vectors will never been same)
    # This is not right solution, we should use another function
    # to obtain the angle
    #if (transformedVec1.normalize() - vec2.normalize()).length > 1e-5:
    #    vec3=CrossVecs(vec2,vec1)     # change of the normal vector   
    #return Quaternion(vec3, AngleBetweenVecs(vec1,vec2))
    # new way, using quaternion features
    #v1=Vector(vec1[0]/vec1.length,vec1[1]/vec1.length,vec1[2]/vec1.length)
    #v2=Vector(vec2[0]/vec2.length,vec2[1]/vec2.length,vec2[2]/vec2.length)
    #print v1,v2
    #q1=Quaternion(0,v1[0],v1[1],v1[2])
    #q2=Quaternion(0,v2[0],v2[1],v2[2])
    #print q1,q2
    #q12=(CrossQuats(q2,q1))
    #q12.negate()
    #q12=Quaternion(q12.w/2,q12.x/2,q12.y/2,q12.z/2)
    #cv=-CrossVecs(v2,v1)
    #cv=Vector(cv[0],cv[1]/2,cv[2]/2)
    #angle=math.degrees(DotVecs(v2,v1))/2
    #q12=Quaternion(cv,angle)
    #print "final",q12
    #return q12
    return qt

def setUpBones(object, boneName, frame):
    # Check proper parameters
    if boneName is None:
        print "Bad parameter: boneName"
        return
    if not (object.getType() == "Armature"):
        print "Bad parameter: object"
        return
    # Try to setup bones
    try:
        # Retrieve bones
        poseBone=object.getPose().bones[boneName] # poseBone object
        bone=object.getData().bones[boneName] # bone object
        # Get the parent bone transformations
        if bone.hasParent():
            parentName=bone.parent.name
            parentPoseBone=object.getPose().bones[parentName]
            parentBone=object.getData().bones[parentName]            
        # Compute rest pose matrix (pose matrix without any rotation)
        # and involve the parent bones transformations
        if bone.hasParent():
            restPoseMatrix = TranslationMatrix(poseBone.loc)*Matrix(bone.matrix["BONESPACE"]).resize4x4()*TranslationMatrix(Vector(bone.head["BONESPACE"]))*TranslationMatrix(Vector(0,parentBone.length,0))*parentPoseBone.poseMatrix            
        else:
            restPoseMatrix = TranslationMatrix(poseBone.loc)*bone.matrix["ARMATURESPACE"]
        # get the names of correspondence empties        
        try:
            [e1name,e2name]=skelDict[boneName]
        except:
            # if not found correspondence, we continue with children and return
            print "no correspondence for bone:",boneName            
            if bone.hasChildren():
                for child in bone.children:
                    setUpBones(object,child.name,frame)
            return
        # get the empties objects
        try:
            e1=Object.Get(e1name)
            e2=Object.Get(e2name)    
        except:
            # if there is a error with finding the empties we return
            print "Error: not found empties:",e1name," or ",e2name
            return
        # get the empties locations
        emptyHead=Vector(e1.getLocation("worldspace")) # this is the head wanted start position
        emptyTail=Vector(e2.getLocation("worldspace")) # and this is the end position
        ### DESCRIPTION HOW TO FIND THE NEEDED ROTATION FOR THE BONE
        ###
        ### head_rest_position = null_vector * rest_pose_matrix
        ### tail_rest_position = (0,bone.length,0) * rest_pose_matrix
        ###
        ### then we have  empty_head_position, empty_tail_position
        ###
        ### and we are looking for such transformation (rotation), that
        ###
        ### head_final_position = null_vector * rotation * rest_pose_matrix
        ### tail_final_position = (0,bone.length,0) * rotation * rest_pose_matrix
        ###
        ### so that
        ###
        ### (tail_final_position-head_final_position)  (empty_tail_position-empty_head_position)
        ### ---------------------------------------- = -----------------------------------------
        ### |tail_final_position-head_final_position|  |empty_tail_position-empty_head_position|
        ###
        ### this means: |e_t_p-e_h_p|(t_f_p-h_f_p)=|t_f_p-h_f_p|(e_t_p-e_h_p)
        ###
        ### so: |e_t_p-e_h_p|(0,bone.length,0) * rotation * rest_pose_matrix = bone.length(e_t_p-e_h_p)
        ###                                                                     ^_ if the pose_matrix doesnt change the length of the bone
        ### (0,bone.length,0) * rotation = bone.length/|e_t_p-e_h_p|(e_t_p-e_t_h)*invert(rest_pose_matrix)
        ###
        ### and becouse it is a rotation, we can ommit the changing of the size and write
        ###
        ### (0, bone.length,0) * rotation = (empty_tail_position-empty_head_position)*invert(rest_pose_matrix)
        ###
        # get the vectors
        eVec=emptyTail-emptyHead
        restVec=Vector(0,bone.length,0)
        finalVec=eVec*restPoseMatrix.invert()
        # USING IPO IS IMPOSSIBLE
        #if boneName in onlyDif:
        #ipo=e1.getIpo()
        #    print ipo.curves
        #    print dir(ipo)
        #    print "IS:", (Ipo.OB_LOCX in ipo)
        #rx=ipo[Ipo.OB_ROTX]
        #ry=ipo[Ipo.OB_ROTY]
        #rz=ipo[Ipo.OB_ROTZ]
        
        #print dir(Ipo)
        #print ipo.curves
        #    print "RX:",rx
        #if boneName in onlyDif:
        #    rotation=Euler((ry[frame]-ry[2])*180/math.pi,(rx[frame]-rx[2])*180/math.pi,(rz[frame]-rz[2])*180/math.pi).toQuat()            
        #else:
        #    rotation=Euler(ry[frame]*180/math.pi,rx[frame]*180/math.pi,rz[frame]*180/math.pi).toQuat()
        #print rx[frame],ry[frame],rz[frame]
        
        # get the needed rotation
        rotation=getQuat(  restVec, finalVec   )        
        # if only difference should be set
        if boneName in onlyDifVecsDict.keys():
            rotation=getQuat( onlyDifVecsDict[boneName], eVec)
        # set the needed rotation
        poseBone.quat = rotation
        # now we try to setup the key for animation and update pose
        try:
            poseBone.insertKey(object,frame,[Pose.ROT])            
            object.getPose().update()
        except:
                print "an error setting the pose key"
        if bone.hasChildren():
            for chb in bone.children:                
                setUpBones(object,chb.name,frame)
    except:
        print "There is not such a bone:",boneName
        
def copyBVH(mySkelet):
    """
    def copyBVH(mySkelet):
        will copy BVH movement, so the bones of mySkelet will copy the movement
        from empties
    """
    action = Blender.Armature.NLA.NewAction()
    action.setActive( mySkelet)
    pose=mySkelet.getPose()
    print "Debug:pose bones:",pose.bones
    # go through all the frames
    scn = Blender.Scene.GetCurrent()
    context = scn.getRenderingContext()
    lastf=context.endFrame()
    for boneName in onlyDif:
        try:
            [e1name,e2name]=skelDict[boneName]
        except:
            # if not found correspondence, we continue with children and return
            print "no correspondence for bone:",boneName            
        # get the empties objects
        try:
            e1=Object.Get(e1name)
            e2=Object.Get(e2name)    
            emptyHead=Vector(e1.getLocation("worldspace")) # this is the head wanted start position
            emptyTail=Vector(e2.getLocation("worldspace")) # and this is the end position
            eVec=emptyTail-emptyHead
            onlyDifVecsDict[boneName]=eVec
        except:
            # if there is a error with finding the empties we return
            print "Error: not found empties:",e1name," or ",e2name        
    for f in range(lastf):
    #for f in range(50):
        print "frame: ",f
        Blender.Set( 'curframe', f)
        # go through all the bones and find these bones, that do not have
        # any parent bones
        for b in pose.bones.keys():            
            if (not mySkelet.getData().bones[b].hasParent()):
                #print mySkelet.getPose().bones[b]
                try:
                    #print mySkelet,"setup:",b
                    setUpBones(mySkelet,b,f)
                except:
                    print "an error during setting bones, check the names of empties"
          
            #pose.update()

    # now we need to also copy the location and rotaion of hips
    # the location needs to be somehow scaled, but how?
    # we try to measure the height of the objects, end 
    # compute the scale factor
    topObj=Object.Get(bvhTop)
    bottomObj=Object.Get(bvhBottom)
    # the height of the skeleton
    h=1.75
    scaleFac = h/(Vector(topObj.getLocation("worldspace"))-Vector(bottomObj.getLocation("worldspace"))).length    
    # the object that we will follows
    rObj=Object.Get(bvhRoot)
    # the location must be must be scaled
    mySkelet.setIpo(rObj.getIpo())
    for t in range(lastf):
        mySkelet.getIpo()[Ipo.OB_LOCX][t]=mySkelet.getIpo()[Ipo.OB_LOCX][t]*scaleFac
        mySkelet.getIpo()[Ipo.OB_LOCY][t]=mySkelet.getIpo()[Ipo.OB_LOCY][t]*scaleFac
        mySkelet.getIpo()[Ipo.OB_LOCZ][t]=mySkelet.getIpo()[Ipo.OB_LOCZ][t]*scaleFac
    mySkelet.getIpo()[Ipo.OB_LOCX].recalc()
    mySkelet.getIpo()[Ipo.OB_LOCY].recalc()
    mySkelet.getIpo()[Ipo.OB_LOCZ].recalc()
    #mySkelet.getIpo()[Ipo.OB_LOCY]
    #mySkelet.getIpo()[Ipo.OB_LOCZ]
    #lastpos=Vector(rObj.getLocation("worldspace"))
    #for f in range(lastf):
    #    Blender.Set( 'curframe', f)
    #    # get the new parameters
    #    newpos=Vector(rObj.getLocation("worldspace"))
    #    rot=rObj.getEuler("worldspace")
    #    to=Vector(mySkelet.getLocation("worldspace"))+scaleFac*(newpos-lastpos)
    #    lastpos=newpos
    #   mySkelet.setLocation( to[0],to[1],to[2])
    #    mySkelet.setEuler(rot)
    #    mySkelet.insertIpoKey(Object.LOCROT)
    
    Blender.Set( 'curframe', 1)
                    