# Blender Python Script for generating Human Skelleton by ISO/IEC FCD 19774 recomendation
# v.1.0

import Blender
import obj_import
import math
from Blender import Armature
from Blender.Mathutils import *

def attachArmature(objects):
    """
    def attachArmature(objects): create vertex groups for the bones and attach armature to mesh
    one of the objects must be a mesh a second armature
    the groups are generated for the vertices with the highest angle between
    head and tail of the bone
    """
    armtr=None
    msh=None
    tmat=None
    for o in objects:
        if o.getType() == "Armature":
            aobj=o
            armtr=o.getData()
        if o.getType() == "Mesh":
            mobj=o
            msh=o.getData(False,True)
            tmat=o.getMatrix()
            print Matrix(tmat).transpose()
    # make the armature parent to mesh
    aobj.makeParentDeform([mobj])
    # create groups
    for bname in armtr.bones.keys():
        msh.addVertGroup(bname)
    # now go through all the vertices and check it with the bones
    minx=100;
    miny=100;
    maxy=0;
    minz=100;
    for v in msh.verts:
        point = (Vector(v.co).resize4D()*tmat).resize3D()
        #point = v.co
        #point = Vector([ point[0]/point[3], point[1]/point[3], point[2]/point[3]]) # cause resize is resizing also x,y,z     
        if point[0]<minx:minx=point[0]
        if point[1]<miny:miny=point[1]
        if point[1]>maxy:maxy=point[1]
        if point[2]<minz:minz=point[2]
        #initial values
        bn=bname
        w=1
        w2=0
        bn2=None
        J=0
        J2=0
        d=10000
        d2=10000
        for [bname,bo] in armtr.bones.items():
            vec1=bo.head['ARMATURESPACE']
            vec2=bo.tail['ARMATURESPACE']
            c=vec2-vec1
            #print point
            #length=math.sqrt(vec1.length*vec1.length+vec2.length*vec2.length)
            vd1=(vec1-point)
            vd2=(vec2-point)
            dist=(vd1.length+vd2.length)#/(vec2-vec1).length
            #length=vd1.length+vd2.length          
            angle=AngleBetweenVecs(Vector(vd1).normalize(),Vector(vd2).normalize())
            angle2=AngleBetweenVecs(-Vector(vd1).normalize(),Vector(c).normalize())
            angleh=AngleBetweenVecs(-Vector(vd1).normalize(),Vector(c).normalize())
            anglet=AngleBetweenVecs(-Vector(vd2).normalize(),-Vector(c).normalize())
            d=(vd2.length+vd1.length)/c.length
            #Jc=1/(1.1*anglet+angleh)#+/90+0.5*d
            Jc=angle#Pi/90+1/math.sqrt(dist)
            # check for the right side of the mesh and bones
            if ((vec1[0]>0) and (point[0]<0)) or ((vec1[0]<0) and (point[0]>0)):
                Jc = 0
            if (J<Jc): # discriminating the too long distance                
                bn=bname
                w=60/angleh
                if w > 1:
                    w=1
                J=Jc                                
        #print point,l
        msh.assignVertsToGroup(bn, [v.index], w, Blender.Mesh.AssignModes.ADD)
        if (w < 1.0) & (armtr.bones[bn].hasParent()):
                msh.assignVertsToGroup(armtr.bones[bn].parent.name, [v.index], (1-w), Blender.Mesh.AssignModes.ADD)
    print minx,miny,maxy,minz   
    
        

def loadMHMesh(file):
    obj_import.load_obj(file,1,0,0.0,0,)


def fitToSkel(object):
    """
    scales a make human mesh to fir the generated skeleton by genSkelMH() function
    """
    # first we need to rotate it correctly
    # and note that this rotation changes z and y 
    bbox = object.getBoundBox()
    print "before:",bbox
    object.setEuler([-math.pi/2,0.0,0.0])
    # Note that the skeleton direction is 
    #   y
    #   ^
    #   |
    #   |
    #   +----> x
    # These values are obtained from skeleton dimensions
    height = 1.7504 # derived from skull bone tail
    width = 2*0.840 # from wrist bone tail
    # deep will be changed as the average of width and height
    bbox = object.getBoundBox()
    print "after:",bbox
    # we will expect that the mesh direction is in the skeleton (armature) direction 
    left = right = bbox[1][0] #x
    top = bottom = bbox[1][2] #y    
    for v in bbox:
        if v[0]<left: left=v[0]
        if v[0]>right: right=v[0]
        if v[2]<bottom: bottom=v[2]
        if v[2]>top: top=v[2]
    # compute the sizes to change
    scaleZ=height/(top-bottom)
    scaleX=width/(right-left)
    scaleY=(scaleX+scaleZ)/2
    object.setSize( scaleX, scaleY, scaleZ)
    # now find the objects position
    bbox = object.getBoundBox()    
    left = bbox[1][0] #x
    bottom = bbox[1][2] #y
    front = bbox[1][1] #z
    for v in bbox:
        if v[0]<left: left=v[0]
        if v[2]<bottom: bottom=v[2]
        if v[1]>front: front=v[1]
    deltaX = -0.840-left*scaleX
    deltaZ = front*scaleY-0.1433 # from midtarsal
    deltaY =  -bottom*scaleZ 
    object.setLocation( deltaX, deltaY, deltaZ)
    
    
def genSkelMH():
    myObj = Blender.Object.New( "Armature")
    mySkel = Blender.Armature.Armature( "HumanoidRoot")
    mySkel.makeEditable()
    # First Bone - must be HumanoidRoot
    newBone = Blender.Armature.Editbone()
    newBone.head = Vector( 0.000, 0.8240, 0.0277)
    newBone.tail = Vector( 0.000, 0.921, -0.080)
    mySkel.bones[ "HumanoidRoot"] = newBone	
    lastBone = newBone
    
    # Sacroiliac
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.000, 0.921, -0.080)
    newBone.tail = Vector( 0.000, 1.057, -0.034)
    mySkel.bones[ "sacroiliac"] = newBone
    newBone.parent = mySkel.bones[ "HumanoidRoot"]
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

    
    # vl5
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.000, 1.057, -0.034)
    newBone.tail = Vector( 0.000, 1.4583, -0.0570)
    mySkel.bones[ "vl5"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

    # vt3  	0.0062 1.4583 -0.0570
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.000, 1.4583, -0.0570)
    #newBone.tail = Vector( 0.000, 1.570, -0.032) joined skullbase
    newBone.tail = Vector( 0.000, 1.7504, 0.000)
    mySkel.bones[ "vt3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

    # skullbase - will not use
    #newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( 0.000, 1.6209, 0.0236)
    #newBone.tail = Vector( 0.000, 1.7504, 0.000)
    #mySkel.bones[ "skullbase"] = newBone
    #newBone.parent = lastBone
    #lastBone = newBone
    #newBone.options = [  Armature.CONNECTED]
    

    #HumanoidRoot_to_l_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = mySkel.bones[ "HumanoidRoot"].tail
    newBone.tail = Vector( 0.096, 0.843, -0.029)
    mySkel.bones[ "HumanoidRoot_to_l_hip"] = newBone
    newBone.parent = mySkel.bones[ "HumanoidRoot"]
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

    # l_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.096, 0.843, -0.029)
    newBone.tail = Vector( 0.065, 0.493, -0.011)
    mySkel.bones[ "l_hip"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_knee
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( 0.065, 0.493, -0.044)
    #newBone.tail = Vector( 0.069, 0.08800, -0.053)
    newBone.head = Vector( 0.065, 0.493, -0.011)
    newBone.tail = Vector( 0.069, 0.091, -0.054)
    mySkel.bones[ "l_knee"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

        # l_ankle
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( 0.069, 0.08800, -0.053)
    #newBone.tail = Vector( 0.069, -0.050, -0.071)
    newBone.head = Vector( 0.069, 0.091, -0.054)   
    newBone.tail = Vector( 0.042, 0.012, 0.18)
    mySkel.bones[ "l_ankle"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]


        # l_midtarsal
    #newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( 0.1086, 0.0001, 0.0368)
    #newBone.tail = Vector( 0.1195, 0.0079, 0.1433)
    #mySkel.bones[ "l_midtarsal"] = newBone
    #newBone.parent = lastBone
    #lastBone = newBone
    #newBone.options = [  Armature.CONNECTED]

    #HumanoidRoot_to_r_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = mySkel.bones[ "HumanoidRoot"].tail
    #newBone.tail = Vector( -0.0961, 0.866, -0.058)
    newBone.tail = Vector( -0.096, 0.843, -0.029)
    mySkel.bones[ "HumanoidRoot_to_r_hip"] = newBone
    newBone.parent = mySkel.bones[ "HumanoidRoot"]
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]


        # r_hip
    newBone = Blender.Armature.Editbone()	                    
    newBone.head = Vector( -0.096, 0.843, -0.029)
    #newBone.head = Vector( -0.096, 0.866, -0.058)
    #newBone.tail = Vector( -0.065, 0.493, -0.044)
    newBone.tail = Vector( -0.065, 0.493, -0.011)
    mySkel.bones[ "r_hip"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

        # r_knee
    newBone = Blender.Armature.Editbone()	
    #newBone.tail = Vector( -0.065, 0.493, -0.044) changed due to anatomy
    #newBone.tail = Vector( -0.065, 0.488, -0.004)
    #newBone.tail = Vector( -0.069, 0.066, -0.042)
    newBone.head = Vector( -0.065, 0.493, -0.011)
    newBone.tail = Vector( -0.069, 0.091, -0.054)   
    mySkel.bones[ "r_knee"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

        # r_ankle
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( -0.069, 0.08800, -0.053)
    #newBone.head = Vector( -0.069, 0.066, -0.042)
    #newBone.tail = Vector( -0.069, -0.05000, -0.071)
    #newBone.tail = Vector( -0.069, 0.0117, 0.156)
    newBone.head = Vector( -0.069, 0.091, -0.054)   
    newBone.tail = Vector( -0.042, 0.012, 0.18)
    mySkel.bones[ "r_ankle"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
    # 0.0974 0.0259 -0.1171
    # r_metatarsal -0.0521 0.0260 0.0127

        # r_midtarsal
    #newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    #newBone.tail = Vector( -0.0883, 0.0134, 0.1383)
    #mySkel.bones[ "r_midtarsal"] = newBone
    #newBone.parent = lastBone
    #lastBone = newBone
    #newBone.options = [  Armature.CONNECTED]

    #vl5 to l_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    newBone.tail = Vector( 0.0820, 1.4488, -0.0353)
    mySkel.bones[ "vl5_to_l_sternoclavicular"] = newBone
    newBone.parent = mySkel.bones[ "vl5"]
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
        
        # l_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0820, 1.4488, -0.0353)
    newBone.tail = Vector( 0.194, 1.434, -0.032)
    mySkel.bones[ "l_sternoclavicular"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
       
    
        # l_shoulder
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.194, 1.434, -0.032)
    newBone.tail = Vector( 0.410, 1.379, -0.062)
    mySkel.bones[ "l_shoulder"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
    
        # l_elbow
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.410, 1.379, -0.062)
    newBone.tail = Vector( 0.659, 1.393, -0.052)
    mySkel.bones[ "l_elbow"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

        # l_wrist
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.659, 1.393, -0.052)
    newBone.tail = Vector( 0.840, 1.391, -0.042)
    mySkel.bones[ "l_wrist"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]

    #vl5 to r_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    newBone.tail = Vector( -0.0694, 1.4600, -0.0330)
    mySkel.bones[ "vl5_to_r_sternoclavicular"] = newBone
    newBone.parent = mySkel.bones[ "vl5"]
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
            
            # r_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0694, 1.4600, -0.0330)
    newBone.tail = Vector(  -0.194, 1.434, -0.032)
    mySkel.bones[ "r_sternoclavicular"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
       
        # r_shoulder
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector(  -0.194, 1.434, -0.032)
    newBone.tail = Vector( -0.410, 1.379, -0.062)
    mySkel.bones[ "r_shoulder"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
    
        # r_elbow
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.410, 1.379, -0.062)
    newBone.tail = Vector( -0.659, 1.393, -0.052)
    mySkel.bones[ "r_elbow"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
    
        # r_wrist
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.659, 1.393, -0.052)
    newBone.tail = Vector( -0.840, 1.391, -0.042)
    mySkel.bones[ "r_wrist"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [  Armature.CONNECTED]
    
    
    
    # Set the deformation distance
    for bone in mySkel.bones.values():
        bone.deformDist = 0.00
    
    # set the automatic inverse kinematics and type of deformation
    mySkel.autoIK=True
    mySkel.envelopes=False
    mySkel.vertexGroups=True

    # update and return armature object	
    myObj.setName( "mySkeleton")
    myObj.link( mySkel)
    sc = Blender.Scene.getCurrent()
    sc.link( myObj)
    mySkel.update()
    return mySkel

def genSkelLevel1():
    myObj = Blender.Object.New( "Armature")
    mySkel = Blender.Armature.Armature( "HumanoidRoot")
    mySkel.makeEditable()
    # First Bone - must be HumanoidRoot
    newBone = Blender.Armature.Editbone()
    newBone.head = Vector( 0.000, 0.8240, 0.0277)
    newBone.tail = Vector( 0.000, 0.9149, 0.0016)
    mySkel.bones[ "HumanoidRoot"] = newBone	
    lastBone = newBone
    
    # Sacroiliac
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.000, 0.9149, 0.0016)
    newBone.tail = Vector( 0.0028, 1.0568, -0.0776)
    mySkel.bones[ "sacroiliac"] = newBone
    newBone.parent = mySkel.bones[ "HumanoidRoot"]
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    
    # vl5
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0028, 1.0568, -0.0776)
    newBone.tail = Vector( 0.0062, 1.4583, -0.0570)
    mySkel.bones[ "vl5"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    # vt3  	0.0062 1.4583 -0.0570
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0062, 1.4583, -0.0570)
    newBone.tail = Vector( 0.0044, 1.6209, 0.0236)
    mySkel.bones[ "vt1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    # skullbase
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0044, 1.6209, 0.0236)
    newBone.tail = Vector( 0.0050, 1.7504, 0.0055)
    mySkel.bones[ "skullbase"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    

    #vt3_to_l_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = mySkel.bones[ "HumanoidRoot"].tail
    newBone.tail = Vector( 0.0961, 0.9124, -0.0001)
    mySkel.bones[ "vl5_to_l_hip"] = newBone
    newBone.parent = mySkel.bones[ "vt3"]
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    # l_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0961, 0.9124, -0.0001)
    newBone.tail = Vector( 0.1040, 0.4867, 0.0308)
    mySkel.bones[ "l_hip"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [Armature.CONNECTED]
    
        # l_knee
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1040, 0.4867, 0.0308)
    newBone.tail = Vector( 0.1101, 0.0656, -0.0736)
    mySkel.bones[ "l_knee"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_ankle
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1101, 0.0656, -0.0736)
    newBone.tail = Vector( 0.1086, 0.0001, 0.038)
    mySkel.bones[ "l_ankle"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]


        # l_midtarsal
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1086, 0.0001, 0.0368)
    newBone.tail = Vector( 0.1195, 0.0079, 0.1433)
    mySkel.bones[ "l_midtarsal"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    #vt3_to_r_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = mySkel.bones[ "HumanoidRoot"].tail
    newBone.tail = Vector( -0.0950, 0.9171, 0.0029)
    mySkel.bones[ "vl5_to_r_hip"] = newBone
    newBone.parent = mySkel.bones[ "vt3"]
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]


        # r_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0950, 0.9171, 0.0029)
    newBone.tail = Vector( -0.0867, 0.4913, 0.0318)
    mySkel.bones[ "r_hip"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_knee
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0867, 0.4913, 0.0318)
    newBone.tail = Vector( -0.0801, 0.0712, -0.0766)
    mySkel.bones[ "r_knee"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_ankle
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0801, 0.0712, -0.0766)
    newBone.tail = Vector( -0.0801, 0.0000, 0.038)
    mySkel.bones[ "r_ankle"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]


        # r_midtarsal
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    newBone.tail = Vector( -0.0883, 0.0134, 0.1383)
    mySkel.bones[ "r_midtarsal"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    #vl5 to l_shoulder
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    newBone.tail = Vector( 0.0820, 1.4488, -0.0353)
    mySkel.bones[ "vl5_to_l_shoulder"] = newBone
    newBone.parent = mySkel.bones[ "sacroiliac"]
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
        
        # l_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0820, 1.4488, -0.0353)
    newBone.tail = Vector( 0.2029, 1.4376, -0.0387)
    mySkel.bones[ "l_sternoclavicular"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
       
    
        # l_shoulder
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2029, 1.4376, -0.0387)
    newBone.tail = Vector( 0.2014, 1.1357, -0.0682)
    mySkel.bones[ "l_shoulder"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_elbow
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2014, 1.1357, -0.0682)
    newBone.tail = Vector( 0.1984, 0.8663, -0.0583)
    mySkel.bones[ "l_elbow"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_wrist
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1984, 0.8663, -0.0583)
    newBone.tail = Vector( 0.2056, 0.6743, -0.0482)
    mySkel.bones[ "l_wrist"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

    #vl5 to r_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    #newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    newBone.tail = Vector( -0.0694, 1.4600, -0.0330)
    mySkel.bones[ "vl5_to_r_sternoclavicular"] = newBone
    newBone.parent = mySkel.bones[ "sacroiliac"]
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
            
            # r_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0694, 1.4600, -0.0330)
    newBone.tail = Vector(  -0.1907, 1.4407, -0.0325)
    mySkel.bones[ "r_sternoclavicular"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [Armature.CONNECTED]
       
        # r_shoulder
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1907, 1.4407, -0.0325)
    newBone.tail = Vector( -0.1949, 1.1388, -0.0620)
    mySkel.bones[ "r_shoulder"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_elbow
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1949, 1.1388, -0.0620)
    newBone.tail = Vector( -0.1959, 0.8694, -0.0521)
    mySkel.bones[ "r_elbow"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_wrist
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1959, 0.8694, -0.0521)
    newBone.tail = Vector( -0.1941, 0.6772, -0.0423)
    mySkel.bones[ "r_wrist"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
    
    
    # Set the deformation distance
    for bone in mySkel.bones.values():
        bone.deformDist = 0.00


    # update and return armature object	
    myObj.setName( "mySkeleton")
    myObj.link( mySkel)
    sc = Blender.Scene.getCurrent()
    sc.link( myObj)
    mySkel.update()
    return mySkel


def genSkelLevel3():
    myObj = Blender.Object.New( "Armature")
    mySkel = Blender.Armature.Armature( "HumanoidRoot")
    mySkel.makeEditable()
    # First Bone - must be HumanoidRoot
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.000, 0.8240, 0.0277)
    newBone.tail = Vector( 0.000, 0.9149, 0.0016)
    mySkel.bones[ "HumanoidRoot"] = newBone	
    lastBone = newBone
    
        # l_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0961, 0.9124, -0.0001)
    newBone.tail = Vector( 0.1040, 0.4867, 0.0308)
    mySkel.bones[ "l_hip"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # l_knee
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1040, 0.4867, 0.0308)
    newBone.tail = Vector( 0.1101, 0.0656, -0.0736)
    mySkel.bones[ "l_knee"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_ankle
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1101, 0.0656, -0.0736)
    newBone.tail = Vector( 0.1086, 0.0001, -0.0368)
    mySkel.bones[ "l_ankle"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_subtalar
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1086, 0.0001, -0.0368)
    newBone.tail = Vector( 0.1086, 0.0001, 0.0368)
    mySkel.bones[ "l_subtalar"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_midtarsal
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1086, 0.0001, 0.0368)
    newBone.tail = Vector( 0.1086, 0.0000, 0.0762)
    mySkel.bones[ "l_midtarsal"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_metatarsal
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1086, 0.0000, 0.0762)
    newBone.tail = Vector( 0.1086, 0.0000, 0.0800)
    mySkel.bones[ "l_metatarsal"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_hip
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0950, 0.9171, 0.0029)
    newBone.tail = Vector( -0.0867, 0.4913, 0.0318)
    mySkel.bones[ "r_hip"] = newBone
    newBone.parent = mySkel.bones[ "HumanoidRoot"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]

        # r_knee
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0867, 0.4913, 0.0318)
    newBone.tail = Vector( -0.0801, 0.0712, -0.0766)
    mySkel.bones[ "r_knee"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_ankle
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0801, 0.0712, -0.0766)
    newBone.tail = Vector( -0.0801, 0.0000, -0.0368)
    mySkel.bones[ "r_ankle"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_subtalar
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0801, 0.0000, -0.0368)
    newBone.tail = Vector( -0.0801, 0.0000, 0.0368)
    mySkel.bones[ "r_subtalar"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_midtarsal
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0801, 0.0000, 0.0368)
    newBone.tail = Vector( -0.0801, 0.0039, 0.0732)
    mySkel.bones[ "r_midtarsal"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_metatarsal
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0801, 0.0039, 0.0732)
    newBone.tail = Vector( -0.0801, 0.0039, 0.0800)
    mySkel.bones[ "r_metatarsal"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # Sacroiliac
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.000, 0.9149, 0.0016)
    newBone.tail = Vector( 0.0028, 1.0568, -0.0776)
    mySkel.bones[ "sacroiliac"] = newBone
    newBone.parent = mySkel.bones[ "HumanoidRoot"]
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vl5
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0028, 1.0568, -0.0776)
    newBone.tail = Vector( 0.0035, 1.0925, -0.0787)
    mySkel.bones[ "vl5"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vl4
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0035, 1.0925, -0.0787)
    newBone.tail = Vector( 0.0041, 1.1276, -0.0796)
    mySkel.bones[ "vl4"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vl3
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0041, 1.1276, -0.0796)
    newBone.tail = Vector( 0.0045, 1.1546, -0.0800)
    mySkel.bones[ "vl3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vl2
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0045, 1.1546, -0.0800)
    newBone.tail = Vector( 0.0048, 1.1912, -0.0805)
    mySkel.bones[ "vl2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
 
        # vl1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0048, 1.1912, -0.0805)
    newBone.tail = Vector( 0.0051, 1.2278, -0.0808)
    mySkel.bones[ "vl1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt12
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0051, 1.2278, -0.0808)
    newBone.tail = Vector( 0.0053, 1.2679, -0.0810)
    mySkel.bones[ "vt12"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt11
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0053, 1.2679, -0.0810)
    newBone.tail = Vector( 0.0056, 1.2848, -0.0822)
    mySkel.bones[ "vt11"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt10
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0056, 1.2848, -0.0822)
    newBone.tail = Vector( 0.0057, 1.3126, -0.0838)
    mySkel.bones[ "vt10"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt9
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0057, 1.3126, -0.0838)
    newBone.tail = Vector( 0.0057, 1.3382, -0.0845)
    mySkel.bones[ "vt9"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt8
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0057, 1.3382, -0.0845)
    newBone.tail = Vector( 0.0058, 1.3625, -0.0833)
    mySkel.bones[ "vt8"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt7
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0058, 1.3625, -0.0833)
    newBone.tail = Vector( 0.0059, 1.3866, -0.0800)
    mySkel.bones[ "vt7"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt6
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0059, 1.3866, -0.0800)
    newBone.tail = Vector( 0.0060, 1.4102, -0.0745)
    mySkel.bones[ "vt6"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt5
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0060, 1.4102, -0.0745)
    newBone.tail = Vector( 0.0061, 1.4320, -0.0675)
    mySkel.bones[ "vt5"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt4
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0061, 1.4320, -0.0675)
    newBone.tail = Vector( 0.0062, 1.4583, -0.0570)
    mySkel.bones[ "vt4"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt3
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0062, 1.4583, -0.0570)
    newBone.tail = Vector( 0.0063, 1.4761, -0.0484)
    mySkel.bones[ "vt3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt2
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0063, 1.4761, -0.0484)
    newBone.tail = Vector( 0.0065, 1.4951, -0.0387)
    mySkel.bones[ "vt2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vt1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0065, 1.4951, -0.0387)
    newBone.tail = Vector( 0.0066, 1.5132, -0.0301)
    mySkel.bones[ "vt1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc7
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.5132, -0.0301)
    newBone.tail = Vector( 0.0066, 1.5357, -0.0143)
    mySkel.bones[ "vc7"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc6
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.5357, -0.0143)
    newBone.tail = Vector( 0.0066, 1.5520, -0.0082)
    mySkel.bones[ "vc6"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc5
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.5520, -0.0082)
    newBone.tail = Vector( 0.0066, 1.5662, -0.0084)
    mySkel.bones[ "vc5"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc4
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.5662, -0.0084)
    newBone.tail = Vector( 0.0066, 1.5800, -0.0103)
    mySkel.bones[ "vc4"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc3
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.5800, -0.0103)
    newBone.tail = Vector( 0.0066, 1.5928, -0.0103)
    mySkel.bones[ "vc3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc2
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.5928, -0.0103)
    newBone.tail = Vector( 0.0066, 1.6144, -0.0034)
    mySkel.bones[ "vc2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # vc1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0066, 1.6144, -0.0034)
    newBone.tail = Vector( 0.0044, 1.6209, 0.0236)
    mySkel.bones[ "vc1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # skullbase
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0044, 1.6209, 0.0236)
    newBone.tail = Vector( 0.0044, 1.6400, 0.0236)
    mySkel.bones[ "skullbase"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0820, 1.4488, -0.0353)
    newBone.tail = Vector( 0.0962, 1.4269, -0.0424)
    mySkel.bones[ "l_sternoclavicular"] = newBone
    newBone.parent = mySkel.bones[ "vt4"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # l_acromioclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.0962, 1.4269, -0.0424)
    newBone.tail = Vector( 0.2029, 1.4376, -0.0387)
    mySkel.bones[ "l_acromioclavicular"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [Armature.HINGE]

        # l_shoulder
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2029, 1.4376, -0.0387)
    newBone.tail = Vector( 0.2014, 1.1357, -0.0682)
    mySkel.bones[ "l_shoulder"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_elbow
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2014, 1.1357, -0.0682)
    newBone.tail = Vector( 0.1984, 0.8663, -0.0583)
    mySkel.bones[ "l_elbow"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_wrist
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1984, 0.8663, -0.0583)
    newBone.tail = Vector( 0.1924, 0.8472, -0.0534)
    mySkel.bones[ "l_wrist"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_thumb1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1924, 0.8472, -0.0534)
    newBone.tail = Vector( 0.1951, 0.8226, 0.0246)
    mySkel.bones[ "l_thumb1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_thumb2
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1951, 0.8226, 0.0246)
    newBone.tail = Vector( 0.1955, 0.8159, 0.0464)
    mySkel.bones[ "l_thumb2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_thumb3
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1955, 0.8159, 0.0464)
    newBone.tail = Vector( 0.1960, 0.8100, 0.0664)
    mySkel.bones[ "l_thumb3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_index0
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1983, 0.8024, -0.0280)
    newBone.tail = Vector( 0.1983, 0.7815, -0.0280)
    mySkel.bones[ "l_index0"] = newBone
    newBone.parent = mySkel.bones[ "l_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # l_index1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1983, 0.7815, -0.0280)
    newBone.tail = Vector( 0.2017, 0.7363, -0.0248)
    mySkel.bones[ "l_index1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_index2
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2017, 0.7363, -0.0248)
    newBone.tail = Vector( 0.2028, 0.7139, -0.0236)
    mySkel.bones[ "l_index2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_index3
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2028, 0.7139, -0.0236)
    newBone.tail = Vector( 0.2038, 0.6939, -0.0224)
    mySkel.bones[ "l_index3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_middle0
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1987, 0.8029, -0.0530)
    newBone.tail = Vector( 0.1987, 0.7818, -0.0530)
    mySkel.bones[ "l_middle0"] = newBone
    newBone.parent = mySkel.bones[ "l_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # l_middle1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1987, 0.7818, -0.0530)
    newBone.tail = Vector( 0.2013, 0.7273, -0.0503)
    mySkel.bones[ "l_middle1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_middle2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2013, 0.7273, -0.0503)
    newBone.tail = Vector( 0.2026, 0.7011, -0.0494)
    mySkel.bones[ "l_middle2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_middle3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.2026, 0.7011, -0.0494)
    newBone.tail = Vector( 0.2039, 0.6761, -0.0485)
    mySkel.bones[ "l_middle3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # l_ring0 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1956, 0.8019, -0.0794)
    newBone.tail = Vector( 0.1956, 0.7815, -0.0794)
    mySkel.bones[ "l_ring0"] = newBone
    newBone.parent = mySkel.bones[ "l_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # l_ring1 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1956, 0.7815, -0.0794)
    newBone.tail = Vector( 0.1973, 0.7287, -0.0777)
    mySkel.bones[ "l_ring1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_ring2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1973, 0.7287, -0.0777)
    newBone.tail = Vector( 0.1983, 0.7045, -0.0767)
    mySkel.bones[ "l_ring2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_ring3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1983, 0.7045, -0.0767)
    newBone.tail = Vector( 0.1993, 0.6800, -0.0757)
    mySkel.bones[ "l_ring3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_pinky0 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1925, 0.8066, -0.1036)
    newBone.tail = Vector( 0.1925, 0.7866, -0.1036)
    mySkel.bones[ "l_pinky0"] = newBone
    newBone.parent = mySkel.bones[ "l_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # l_pinky1
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1925, 0.7866, -0.1036)
    newBone.tail = Vector( 0.1938, 0.7452, -0.1024)
    mySkel.bones[ "l_pinky1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_pinky2
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1938, 0.7452, -0.1024)
    newBone.tail = Vector( 0.1948, 0.7277, -0.1017)
    mySkel.bones[ "l_pinky2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # l_pinky3
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( 0.1948, 0.7277, -0.1017)
    newBone.tail = Vector( 0.1958, 0.7027, -0.1010)
    mySkel.bones[ "l_pinky3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_sternoclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0694, 1.4600, -0.0330)
    newBone.tail = Vector( -0.0836, 1.4281, -0.0401)
    mySkel.bones[ "r_sternoclavicular"] = newBone
    newBone.parent = mySkel.bones[ "vt4"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # r_acromioclavicular
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.0836, 1.4281, -0.0401)
    newBone.tail = Vector( -0.1907, 1.4407, -0.0325)
    mySkel.bones[ "r_acromioclavicular"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_shoulder
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1907, 1.4407, -0.0325)
    newBone.tail = Vector( -0.1949, 1.1388, -0.0620)
    mySkel.bones[ "r_shoulder"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_elbow
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1949, 1.1388, -0.0620)
    newBone.tail = Vector( -0.1959, 0.8694, -0.0521)
    mySkel.bones[ "r_elbow"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_wrist
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1959, 0.8694, -0.0521)
    newBone.tail = Vector( -0.1899, 0.8502, -0.0473)
    mySkel.bones[ "r_wrist"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_thumb1 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1899, 0.8502, -0.0473)
    newBone.tail = Vector( -0.1874, 0.8256, 0.0306)
    mySkel.bones[ "r_thumb1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_thumb2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1874, 0.8256, 0.0306)
    newBone.tail = Vector( -0.1864, 0.8190, 0.0506)
    mySkel.bones[ "r_thumb2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_thumb3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1864, 0.8190, 0.0506)
    newBone.tail = Vector( -0.1854, 0.8134, 0.0706)
    mySkel.bones[ "r_thumb3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_index0 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1961, 0.8055, -0.0218)
    newBone.tail = Vector( -0.1961, 0.7846, -0.0218)
    mySkel.bones[ "r_index0"] = newBone
    newBone.parent = mySkel.bones[ "r_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]
    
        # r_index1 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1961, 0.7846, -0.0218)
    newBone.tail = Vector( -0.1954, 0.7393, -0.0185)
    mySkel.bones[ "r_index1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_index2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1954, 0.7393, -0.0185)
    newBone.tail = Vector( -0.1945, 0.7169, -0.0173)
    mySkel.bones[ "r_index2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]
    
        # r_index3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1945, 0.7169, -0.0173)
    newBone.tail = Vector( -0.1935, 0.6945,-0.0151)
    mySkel.bones[ "r_index3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_middle0 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1972, 0.8060, -0.0468)
    newBone.tail = Vector( -0.1972, 0.7849, -0.0468)
    mySkel.bones[ "r_middle0"] = newBone
    newBone.parent = mySkel.bones[ "r_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]

        # r_middle1 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1972, 0.7849, -0.0468)
    newBone.tail = Vector( -0.1950, 0.7304, -0.0441)
    mySkel.bones[ "r_middle1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]


        # r_middle2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1950, 0.7304, -0.0441)
    newBone.tail = Vector( -0.1939, 0.7042, -0.0432)
    mySkel.bones[ "r_middle2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_middle3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1939, 0.7042, -0.0432)
    newBone.tail = Vector( -0.1928, 0.6780, -0.0423)
    mySkel.bones[ "r_middle3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_ring0 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1951, 0.8049, -0.0732)
    newBone.tail = Vector( -0.1951, 0.7845, -0.0732)
    mySkel.bones[ "r_ring0"] = newBone
    newBone.parent = mySkel.bones[ "r_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]

        # r_ring1 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1951, 0.7845, -0.0732)
    newBone.tail = Vector( -0.1920, 0.7318, -0.0716)
    mySkel.bones[ "r_ring1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_ring2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1920, 0.7318, -0.0716)
    newBone.tail = Vector( -0.1908, 0.7077, -0.0706)
    mySkel.bones[ "r_ring2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_ring3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1908, 0.7077, -0.0706)
    newBone.tail = Vector( -0.1896, 0.6848, -0.0696)
    mySkel.bones[ "r_ring3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_pinky0 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1926, 0.8096, -0.0975)
    newBone.tail = Vector( -0.1926, 0.7896, -0.0975)
    mySkel.bones[ "r_pinky0"] = newBone
    newBone.parent = mySkel.bones[ "r_wrist"]
    lastBone = newBone
    newBone.options = [Armature.HINGE]

        # r_pinky1 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1926, 0.7896, -0.0975)
    newBone.tail = Vector( -0.1902, 0.7483, -0.0963)
    mySkel.bones[ "r_pinky1"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_pinky2 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1902, 0.7483, -0.0963)
    newBone.tail = Vector( -0.1908, 0.7240, -0.0960)
    mySkel.bones[ "r_pinky2"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]

        # r_pinky3 
    newBone = Blender.Armature.Editbone()	
    newBone.head = Vector( -0.1908, 0.7240, -0.0960)
    newBone.tail = Vector( -0.1914, 0.6997, -0.0957)
    mySkel.bones[ "r_pinky3"] = newBone
    newBone.parent = lastBone
    lastBone = newBone
    newBone.options = [ Armature.CONNECTED]



    # update and return armature object	
    myObj.setName( "mySkeleton")
    myObj.link( mySkel)
    sc = Blender.Scene.getCurrent()
    sc.link( myObj)
    mySkel.update()
    return mySkel


#generateSkeleton() # FOR TESTING PURPOSE ONLY - COMMENT THIS WHEN USING FROM ANOTHER SCRIPT

