# This function renders Object and returns its matrix
# with the polygon indexes and z buffer
# so the matrix is width*height*[index,zcoord]
from Blender import *
import numpy
from scipy import *
from types import *
from rahat.poly import *
from rahat.mymatrix import *

# This function transform list of vertices by using given matrix
def transformVertices(vertices,M):        
    T=[ [ M[0].x, M[0].y, M[0].z, M[0].w], 
        [ M[1].x, M[1].y, M[1].z, M[1].w],
        [ M[2].x, M[2].y, M[2].z, M[2].w],
        [ M[3].x, M[3].y, M[3].z, M[3].w]]
    vs = list()
    for v in vertices:
        vec = [ [v[0]], [v[1]], [v[2]], [1.0]]
        vec = mm(T,vec)
        vec = [vec[0][0]/vec[3][0],vec[1][0]/vec[3][0],vec[2][0]/vec[3][0]]
        vs.append(vec)    
    #return vs
    return vertices
    

# This function checks proper face orientation - to the camera
def isTowardsCam(Pmat, normal):
    n = projectVertex(Pmat, normal)     
    if n[2] < 0.0:
        return TRUE
    return FALSE

# This function renders Blender Mesh Object into Matrix with 
# Polygon index and z coordinate
def renderObj(BlenderObject, Pmatrix, width, height):
    # Proper arguments test
    if BlenderObject.getType() != "Mesh":
        print type(BlenderObject)
        print "Wrong argument! - renderObj(BlenderObject, Pmatrix, width, height)"
        print "                  BlenderObject must be NMeshType!"
        return None
    if type(width) != IntType:
        print type(width)
        print "Wrong argument! - renderObj(BlenderObject, Pmatrix, width, height)"
        print "                  width must be IntType!"
        return None
    if type(height) != IntType:
        print type(height)
        print "Wrong argument! - renderObj(BlenderObject, Pmatrix, width, height)"
        print "                  height must be IntType!"
        return None
    # Rendering
    mesh = BlenderObject.getData()
    renderdata = [zeros((width,height)),zeros((width,height),double)]
    polygons = mesh.faces
    index=0
    for p in polygons:
        #if isTowardsCam(Pmatrix,transformVertices( [p.no], BlenderObject.getMatrix())[0]):
            # We should be using p.index if polygon is MFace object
            # but this is not working becouse we have NMesh object
            # so our own index is used
            #print "DEBUG:"
            #print "verts: "
            #print transformVertices( p.v, BlenderObject.getMatrix() )
            #print "Pmat: "
            #print Pmatrix
            #print "Projected"
            #print projectVertex( Pmatrix,transformVertices( p.v, BlenderObject.getMatrix() )[0])
            #print projectVertex( Pmatrix,transformVertices( p.v, BlenderObject.getMatrix() )[1])
            #print projectVertex( Pmatrix,transformVertices( p.v, BlenderObject.getMatrix() )[2])
            draw3DPoly( transformVertices( p.v, BlenderObject.getMatrix() ), Pmatrix, renderdata, index)
            index=index+1
            print index
    return renderdata

# This function returns Blender.Image object from matrix
def matToImage(matrix):
    im=Image.New("Matrix",size(matrix[0],0),size(matrix[0],1),24)
    print "Matrix:",size(matrix[0],0),size(matrix[0],1)
    for x in range(0, size(matrix[0],0)):
        for y in range(0, size(matrix[0],1)):
            if (matrix[0][x,y] != 0):
                if ((matrix[0][x,y])%2 == 0) :
                    im.setPixelI(x, y, (0, (10*matrix[0][x,y])%255, (abs(matrix[1][x,y]))%255, 255)) 
                else:
                    im.setPixelI(x, y, ((10*matrix[0][x,y])%255, (abs(matrix[1][x,y]))%255, 0, 255)) 
                print matrix[1][x,y]
            else:
                im.setPixelI(x,y,(255,255,255,255))
    return im