#!BPY
"""
Name: 'Animation and Texturize Tools'
Blender: 240
Group: 'Misc'
Tooltip: 'Tools to create humanoid animations for multiple cameras'
"""

__author__ = "Mazany Ondrej"
__url__ = "http://cmp.felk.cvut.cz/~mazano1/"
__version__ = "v.0.1"
__email__ = "mazany.o@seznam.cz"
__bpydoc__ = """\
-------------------------------------------------------------------
    This is the main script to be run from Blender 
-------------------------------------------------------------------
"""
#------------------------------------------------------------------
# - CODE - CODE - CODE - CODE - CODE - CODE - CODE - CODE - CODE -
#------------------------------------------------------------------


# Loading Blender Modules
from Blender import *
from Blender.Object import *
from Blender.Mathutils import *
from Blender.Window import *

# Setting the path to the python files
# in windows you use: SET bpypath=c:\path_to_python_files
# in linux you run: bpypath=/path_to_python_files blender -P run.py
import sys
import os
bpypath = os.getenv("bpypath") # append the OS Enviroment variable bpypath
sys.path.append( bpypath)      # to path
print "------------------------------------------------------------------------"
print "Runing the main script of the Animation and Texturize tools"
print "Path to the modules was set to:", bpypath
print "------------------------------------------------------------------------"



# Importing own modules
from rahat.calibim import *
from rahat.poly import *
from rahat.renderobj import *
from rahat.calibcam import *
from rahat.parsecam import *
from rahat.glrender import *
from rahat.attachtex import *
from rahat.parseanim import *
from rahat.bfunc import *
from rahat.fitobj import *
from rahat.genskel import *
from rahat.copybvh import *
from rahat.exportanim import *

# Global variables
LOADCAMEVNT = 1     # event id
EXPORTCAMEVNT = 2   # event id
LOADANIMEVNT = 3    # event id
EXPORTPMATEVNT = 4  # event id
NEXTCAMEVNT = 5     # event id
RENDERTOIMGEVNT = 6 # event id
ADAPTEVNT = 7       # event id
TESTUVEVNT = 8      # event id
ASSINGPEVNT = 9     # event id
TEXTURIZEVNT = 10   # event id
TESTBTNEVNT = 11    # event id
GENERSKELEVNT = 12  # event id
FITTOSKELEVNT = 13  # event id
IMPORTMHEVNT = 14   # event id
ATCHSKELEVNT = 15   # event id
COPYBVHEVNT = 16    # event id
RENDEREVNT = 17     # event id
EXPORTEVNT = 18     # event id
cameras = list()    # list of cameras in the scene
calibimgs = list()  # list of calibrated images
animation = None    # animation
imagefile = None    # name of the file with image




# Draw GUI
def drawGUI():
    """
    def drawGUI(): this function is registered to draw the GUI
    """
    i = 1
    area = Window.GetAreaSize()
    Draw.PushButton("Load and Setup Camera", LOADCAMEVNT, 0, area[1]-i*22, 200, 20, "Loads camera settings from *.cam file")
    i = i+1
    Draw.PushButton("Export Camera Settings", EXPORTCAMEVNT, 0, area[1]-i*22, 200, 20, "Export camera settings to *.cam file")
    i = i+1
    Draw.PushButton("Export Camera Projection Matrix", EXPORTPMATEVNT, 0, area[1]-i*22, 200, 20, "Export projective matrix to *.P ascii file")
    i = i+1
    Draw.PushButton("Get next camera", NEXTCAMEVNT, 0, area[1]-i*22, 200, 20, "Loads image and the Projection Matrix from file")
    i = i+1    
    # Draw.Space
    i = i+1
    Draw.PushButton("Load and Setup animation", LOADANIMEVNT, 0, area[1]-i*22, 200, 20, "Loads animation from *.xml file")
    i = i+1
    # Draw.Space
    i = i+1
    Draw.PushButton("Assing P matrix to Image", ASSINGPEVNT, 0, area[1]-i*22, 200, 20, "Loads image and the Projection Matrix from file")
    i = i+1
    Draw.PushButton("Render Object to Images", RENDERTOIMGEVNT, 0, area[1]-i*22, 200, 20, "Renders selected objects into calibrated images")
    i = i+1
    Draw.PushButton("Adapt Object to Images", ADAPTEVNT, 0, area[1]-i*22, 200, 20, "Renders selected objects into calibrated images")
    i = i+1
    Draw.PushButton("Test Object UVs in Images", TESTUVEVNT, 0, area[1]-i*22, 200, 20, "Renders selected objects into calibrated images")
    i = i+1
    # Draw.Space
    i = i+1
    Draw.PushButton("Attach Images as Texture", TEXTURIZEVNT, 0, area[1]-i*22, 200, 20, "Attaches images to selected object as a textures")
    #BGL.glRasterPos2i(225, 44)
    #Draw.Text(loadstatus)    
    i = i+1
    # Draw.Space
    i = i+1
    Draw.PushButton("Test Button", TESTBTNEVNT, 0, area[1]-i*22, 200, 20, "Debug button for testing")
    i = i+1
    # Draw.Space
    i = i+1
    Draw.PushButton("Generate Skeleton", GENERSKELEVNT, 0, area[1]-i*22, 200, 20, "Generates Humanoid Skeleton")
    i = i+1
    Draw.PushButton("Fit to Skeleton", FITTOSKELEVNT, 0, area[1]-i*22, 200, 20, "Fits the mesh sizes and loaction to match the generated skeleton")
    i = i+1
    Draw.PushButton("Import MH Mesh", IMPORTMHEVNT, 0, area[1]-i*22, 200, 20, "Imports MakeHuman exported mesh from .obj file")
    i = i+1
    Draw.PushButton("Attach Skeleton to Mesh", ATCHSKELEVNT, 0, area[1]-i*22, 200, 20, "Select armature and mesh objects, then this button will create vertex groups and attaches armature to mesh")
    i = i+1
    Draw.PushButton("Copy BVH movement", COPYBVHEVNT, 0, area[1]-i*22, 200, 20, "Copy the movement from empties imported from BVH file")
    i = i+1
    # Draw.Space
    i = i+1
    Draw.PushButton("Render for each camera", RENDEREVNT, 0, area[1]-i*22, 200, 20, "Renders animation for each camera")
    i = i+1
    # Draw.Space
    i = i+1
    Draw.PushButton("Export animation", EXPORTEVNT, 0, area[1]-i*22, 200, 20, "Exports vertices and bones locations during animation")
    return

# Load Camera
def loadCam( file):
    """
    def loadCam( file): this is event handler, called by file selector after
    choosing the *.cam file. It loads and setups a camera in blender.
    """
    cam = loadCamera( file)
    cam.setUp()
    global cameras
    cameras.append( cam)
    print "loaded camera:",cam," from:",file
    return cam

# Export Camera
def exportCam( file):
    """
    def exportCam( file): this is event handler called by path selector after
    choosing the path for *.cam file. It exports the camera settings to file.
    """
    # first we need to find object with camera settings
    for object in GetSelected():
        if object.getType() == "Camera":
            cam = CalibCam( object )
            saveCamera( cam, file)
            print "exported camera:",cam," to:",file


# Export Projective Matrix
def exportPMat( file):
    """
    def exportPMat( file): this is event handler called by path selector after
    choosing the path for *.P file. It exports the camera projective matrix to file.
    """
    # first we need to find object with camera settings
    for object in GetSelected():
        if object.getType() == "Camera":
            cam = CalibCam( object )
            writePMatrix( file, cam)
            print "exported projective P matrix of ",cam," camera to:",file


# Load Animation
def loadAnim( file):
    """
    def loadAnim( file): this is event handler, called by file selector after
    choosing the *.xml file. It loads and setups the animation in blender.
    """
    parseanim( file)
    setUpFrames()
    print "loaded animation from xml file:",file

# Load image
def getNextCam():
    """
    def getNextCam(): this is event handler.
    It set the next camera from cameras list active.
    """
    # otain blender objects
    objects = Object.Get()
    # obtain current scene
    cur = Scene.getCurrent()  
    # get the active camera
    curCam = cur.getCurrentCamera()
    # look for next camera in the scene
    isNext = False
    isFirst = True
    nextCam = curCam
    for o in objects:
        if o.getType() == "Camera":
            if isFirst:
                nextCam = o
                isFirst = False
            if isNext:
                nextCam = o
                isNext = False
            if o.getName() == curCam.getName():
                isNext = True
    if nextCam:
        cur.setCurrentCamera( nextCam)
        curCam.select(False)
        nextCam.select(True)
        Window.CameraView()
        print "Camera selected:",nextCam.getName()
    return nextCam
    
# Load P matrix
def assingP( file):
    """
    def assingPt( file): this is event handler, called by file selector.
    It creates CalibIm object and associate it to currently displayed image
    and attaches to it P matrix.
    """
    CI = CalibIm()
    CI.img = Image.GetCurrent()
    CI.loadMatrix( file)
    CI.toBitmap()
    global calibimgs    
    calibimgs.append( CI)
    print "Assigned P matrix from:",file," to image:",CI.img
    print "Matrix:"
    print CI.getPMatrix()
    return CI


# Render objects to images
def renderToImg():
    """
    def renderToImg(): this is called from event handler. It renders all selected
    objects into calibrated images
    """
    for object in GetSelected():
        if object.getType() == "Mesh":
            for CI in calibimgs:
                CI.renderToImage( object)
                print "Mesh object :",object," rendered to:",CI.img
    return object
    
# Adapt mesh to images
def adaptToImgs():
    """   
    def adaptToImgs(): this is called from event handler. adapts mesh vertices 
    to the images
    """
    for object in GetSelected():
        if object.getType() == "Mesh":
            fitMesh( object, calibimgs,1)
            print "Mesh object:", object," adapted to images:", calibimgs
    return object

# Test Object UVs in images
def testUV():
    """   
    def testUV(): this is called from event handler. prints uv coordinates 
    of selected object in loaded images
    """
    for object in GetSelected():
        if object.getType() == "Mesh":
            print "Objects transformation matrix:"
            print Matrix(object.getMatrix()).transpose()
            for v in getDeformedMeshData(object).verts:
                cp = Vector( v.co)
                print "Vertex: ",cp.resize4D()
                for c in calibimgs:  
                    #initRender()
                    c.initRendering()
                    # and init model transformations
                    glMatrixMode( GL_MODELVIEW)
                    glMultMatrixd( matToBuf( Matrix(object.getMatrix()).transpose()))
                    ver = object.getMatrix() * cp.resize4D()
                    print "Transformed vertex: ", ver                  
                    uv = c.project( ver[0], ver[1], ver[2])
                    print c.getImage().getName(),": ComputerVision uv = ", uv
                    uv = c.getUVFromView( cp)
                    print c.getImage().getName(),": Blenders coord uv = ", uv

# Test Object UVs in images
def texturize():
    """   
    def texturize(): this is called from event handler. assings uv coordinates 
    of selected object in loaded images and attaches them as textures to object
    """
    for object in GetSelected():
        if object.getType() == "Mesh":
            #print "UV coords before texturing:"
            #for f in getDeformedMeshData(object).faces:
            #    print f.uv
            print "Texturing mesh object:",object
            attachTex( object, calibimgs)
            #print "UV coords after texturing:"
            #for f in getDeformedMeshData(object).faces:
                #print f.uv[0][0],f.uv[0][1],f.uv[1][0],f.uv[1][1]
                #f.uv[0]=[0.5,0.5]
                #f.uv[0][0] = f.uv[0][0]/2
                #f.uv[0][1] = f.uv[0][1]/2
                #f.uv[1][0] = f.uv[1][0]/2
                #f.uv[1][1] = f.uv[1][1]/2
                #object.getData().update()
                #print f.uv
    return object

# fits a mesh to generated skeleton
def fitMeshToSkel():
    """   
    def fitMesh2Skel(): this is called from event handler.
    fit a make human mesh to generated skeleton
    """
    for object in GetSelected():
        if object.getType() == "Mesh":
            fitToSkel( object)
            print "Mesh object:",object," set to fit the skeleton"
    return object

# create vertex groups for the bones
def attachSkeleton():
    """
    def attachSkeleton(): this is called from event handler.
    """
    attachArmature(GetSelected())
    print "Selected mesh was attached to selected Armature"


def cpBVH():
    """
    def cpBVH(): this is called from event handler. It copies BVH movement"
    """
    for object in GetSelected():
        if object.getType() == "Armature":
            copyBVH(object)
            print "Armature object:",object," was set to copy the movements of BVH empties"
    
def rndr():
    """
    def cpBVH(): this is called from event handler. It renders animation for each camera
    """
    outputpath=Draw.PupStrInput("output path:", "\\", 100)
    runRender(outputpath,cameras)
    print "Animation rendered to:",outputpath,"for cameras:",cameras
    


def exportAnim():
    """
    def cpBVH(): this is called from event handler. 
    It exports vertices and bones location to the output path
    """
    outputpath=Draw.PupStrInput("output path:", "\\", 100)
    for object in GetSelected():
        if object.getType() == "Armature":    
            exportBones(object,outputpath)
            print "Animation exported to:",outputpath,"for armature object:",object
        if object.getType() == "Mesh":    
            exportVertices(object,outputpath)
            print "Animation exported to:",outputpath,"for mesh object:",object
    
# Udalosti tlacitek    
def button_evt(event):
    """
    def button_evt(event):  this is event handler called by button events
    """
    if (event == LOADCAMEVNT):
        # Choose file and load it and setup the camera
        Window.FileSelector( loadCam, "Choose *.cam file")
    if (event == EXPORTCAMEVNT):
        # Choose path and save camera settings
        Window.FileSelector( exportCam, "Choose *.cam file")
    if (event == LOADANIMEVNT):
        # Choose xml file with animation
        Window.FileSelector( loadAnim, "Choose *.xml file")        
    if (event == EXPORTPMATEVNT):
        # Choose xml file with animation
        Window.FileSelector( exportPMat, "Choose *.P file")        
    if (event == NEXTCAMEVNT):
        # sets the next camera active
        getNextCam()
    if (event == RENDERTOIMGEVNT):
        # Render selected objects to images
        renderToImg()        
    if (event == ADAPTEVNT):
        # Adapts objects to images
        adaptToImgs()        
    if (event == TESTUVEVNT):
        # Adapts objects to images
        testUV()        
    if (event == ASSINGPEVNT):
        # Assings P matrix to the Image already displayed
        Window.FileSelector( assingP, "Choose *.P file")      
    if (event == TEXTURIZEVNT):
        # Assings P matrix to the Image already displayed
        texturize()     
    if (event == TESTBTNEVNT):
        # Test functions
        test()     
    if (event == GENERSKELEVNT):
        # Generate Skeleton
        genSkelMH()     
    if (event == FITTOSKELEVNT):
        # fits to skeleton
        fitMeshToSkel()
    if (event == IMPORTMHEVNT):
        # imports make human mesh
        Window.FileSelector( loadMHMesh, "Choose *.obj file")  
    if (event == ATCHSKELEVNT):
        # create vertex groups and attach skeleton
        attachSkeleton() 
    if (event == COPYBVHEVNT):
        # create vertex groups
        cpBVH() 
    if (event == RENDEREVNT):
        # render for each camera
        rndr()
    if (event == EXPORTEVNT):
        # render for each camera
        exportAnim()
                 
        
def test():
    # this will be used to test the vertex armature deformations
    for o in GetSelected():
        if o.getType() == "Mesh":
            print "Object's parent:",o.getParent()
            arm = o.getParent()
            print "Object's pose:",arm.getPose()
            pos = arm.getPose()                        
            nm = o.getData()
            nmr = NMesh.GetRawFromObject(o.getName())
            oMat = o.getMatrix()
            aMat = arm.getMatrix()
            print "Vertex groups:",nm.getVertGroupNames()
            for ver in nm.verts:
                print "Vertex",ver.index," is affected by:",nm.getVertexInfluences(ver.index)
                print "Vertex deformed location=",nmr.verts[ver.index].co
                #print "Vertex worldspace location:",Vector(nmr.verts[ver.index].co).resize4D()*o.getMatrix()                
                lco = Vector(ver.co).resize4D()
                wco = Vector(ver.co).resize4D() * oMat
                print "local co=",lco
                print "world co=",wco
                contrib = 0.0
                # this is the change to the default
                vec=Vector(0,0,0,1)
                for bd in nm.getVertexInfluences(ver.index):
                    boneName = bd[0]
                    weight = bd[1]
                    contrib = contrib + weight
                    boneMat = arm.getData().bones[ boneName].matrix['ARMATURESPACE']
                    #blco = lco*Matrix(boneMat).invert()
                    #bwco = wco*Matrix(boneMat).invert()
                    aco = wco*Matrix(aMat).invert()
                    print "Vertex in armature space:",aco
                    bco = aco*Matrix(boneMat).invert()
                    print "Vertex in bone space:",bco
                    #print "blco=lco*BoneMat^-1:",blco
                    #print "bwco=wco*BoneMat^-1:",bwco
                    # get the coordinates in bone space
                    bnco = lco*Matrix(boneMat).invert()
                    poseMat = pos.bones[ boneName].poseMatrix
                    localMat = pos.bones[ boneName].localMatrix
                    pco = bco*poseMat
                    print "Affected by pose in armature space:",pco
                    vec[0]=vec[0]+(pco[0]-aco[0])*weight
                    vec[1]=vec[1]+(pco[1]-aco[1])*weight
                    vec[2]=vec[2]+(pco[2]-aco[2])*weight
                    #print "Change againts the default:",vec
                    #print poseMat
                    #print localMat
                    #newco = newco + (newco*poseMat - newco)*bd[1]
                    #defco = weight*bnco*poseMat*arm.getMatrix()
                    #defloc = weight*bnco*localMat*arm.getMatrix()
                    # get the deformation
                    #defco = bnco*poseMat*arm.getMatrix()
                    #vec = (bnco*localMat-bnco)*weight                    
                if contrib > 0.0:
                    vec[0]=vec[0]/contrib
                    vec[1]=vec[1]/contrib
                    vec[2]=vec[2]/contrib
                print "Total change in armature space:",vec
                finco = Vector(0,0,0,1)
                finco[0] = aco[0]+vec[0]
                finco[1] = aco[1]+vec[1]
                finco[2] = aco[2]+vec[2]
                print "The final loc in armature space:",finco
                finw= finco*aMat
                print "The final in world space:",finw
                mloc= finw*Matrix(oMat).invert()
                print "In the local mesh space:",mloc
                print "should equals to:",nmr.verts[ver.index].co
                print "-------------------------------------------------------------------------------"
                #postco = bnco
                #print "blco*localMat=",blco*localMat
                #print "blco*localMat*ArmMat=",blco*localMat*arm.getMatrix()
                #print "blco*localMat*ObjMat=",blco*localMat*o.getMatrix()
                #print "blco*localMat*ObjMat*ArmMat=",blco*localMat*o.getMatrix()*arm.getMatrix()
                #print "bwco*localMat=",bwco*localMat
                #print "bwco*localMat*ArmMat=",bwco*localMat*arm.getMatrix()
                #print "bwco*localMat*ObjMat=",bwco*localMat*o.getMatrix()
                #print "bwco*localMat*ObjMat*ArmMat=",bwco*localMat*o.getMatrix()*arm.getMatrix()
                #print "blco*localMat+lco=",blco*localMat+lco
                #print "blco*localMat*ArmMat+lco=",blco*localMat*arm.getMatrix()+lco
                #print "blco*localMat*ObjMat+lco=",blco*localMat*o.getMatrix()+lco
                #print "blco*localMat*ObjMat*ArmMat+lco=",blco*localMat*o.getMatrix()*arm.getMatrix()+lco
                #print "bwco*localMat+lco=",bwco*localMat+lco
                #print "bwco*localMat*ArmMat+lco=",bwco*localMat*arm.getMatrix()+lco
                #print "bwco*localMat*ObjMat+lco=",bwco*localMat*o.getMatrix()+lco
                #print "bwco*localMat*ObjMat*ArmMat+lco=",bwco*localMat*o.getMatrix()*arm.getMatrix()+lco
                #print "Vertex deformed location:",postco.resize3D()
                #print "Vec:",vec
##        print "---------------------------------"
##        print "Object:",o.getName()
##        print o.RotX,o.RotY,o.RotZ
##        print "Euler:",o.getEuler()
##        print "Matrix:"
##        print o.getEuler().toMatrix()
##        print "world:"
##        print o.getMatrix()
##        if o.getType() == "Camera":
##            c=CalibCam(o)
##            print c.getP()
##    for o in GetSelected():
##        if o.getType() == "Armature":
##            for b in o.getPose().bones.values():
##                bn=getDeformedMeshData(o).bones[b.name]
##                if bn.hasParent():
##                    parentBN=bn.parent
##                #for op in bn.options:
##                    #if op == Armature.BONE_SELECTED:
##                ########### KEEEEP THIS ###########################
##                ######### THIS SAYS HOW TO COMPUTE BONES ##########
##                ###################################################
##                print "---------------",bn.name,"--------------------"
##                print bn.head["BONESPACE"]
##                print bn.tail["BONESPACE"]
##                print "bone.matrix[BONESPACE]"
##                print bn.matrix["BONESPACE"]
##                # bonematrix = roll*orientation   (of y axis)
##                bonematrix = RotationMatrix( bn.roll["BONESPACE"],3,"y")*getQuat(Vector(0,0,0),Vector(0,1,0),Vector(bn.head["BONESPACE"]),Vector(bn.tail["BONESPACE"])).toMatrix()
##                print "bonematrix"
##                print bonematrix
##                print "bone.matrix[ARMATURESPACE]"
##                print bn.matrix["ARMATURESPACE"]
##                # armaturematrix = parent_armaturematrix*TranslationY(parent_tail_length)*bonehead*bonematrix
##                if parentBN:
##                    armaturematrix=bonematrix.resize4x4()*TranslationMatrix(Vector(bn.head["BONESPACE"]))*TranslationMatrix(Vector(0,parentBN.length,0))*parentBN.matrix["ARMATURESPACE"]
##                else:
##                    armaturematrix=bonematrix.resize4x4()*TranslationMatrix(Vector(bn.head["BONESPACE"]))
##                print "armaturematrix"
##                print armaturematrix
##                print "bone.poseMatrix"
##                print b.poseMatrix
##                # posematrix = parent_pose*translationy(parent_tail_lenght)*translation(head)*bonematrix*translation*rotation
##                if parentBN:
##                    parentPS=o.getPose().bones[parentBN.name]
##                    posematrix = b.quat.toMatrix().resize4x4()*TranslationMatrix(b.loc)*bonematrix*TranslationMatrix(Vector(bn.head["BONESPACE"]))*TranslationMatrix(Vector(0,parentBN.length,0))*parentPS.poseMatrix
##                else:
##                    posematrix = b.quat.toMatrix().resize4x4()*TranslationMatrix(b.loc)*armaturematrix
##                print "posematrix"
##                print posematrix
##                print "poseBone.head, tail"
##                print b.head
##                print b.tail
##                print "print pose head, tail"
##                print Vector(0,0,0,1)*posematrix
##                print Vector(0,bn.length,0,1)*posematrix
##                print "localmatrix"
##                print b.localMatrix
##                print "quat"
##                print b.quat.toMatrix()
##                print "quat*armature"
##                print armaturematrix*Matrix(b.quat.toMatrix()).resize4x4()
##                print "loc"
##                print b.loc
##                parentBN=None
##            for b in o.getData().bones.values():
##                print "bone:",b.name
##                print "head:",b.head["ARMATURESPACE"],b.head["BONESPACE"]
##                print "tail:",b.tail["ARMATURESPACE"],b.tail["BONESPACE"]
##                print "matrix:",b.matrix
##                print "roll,length:",b.roll,b.length
                
                
##    Window.CameraView()
##    print GetScreens()
##    print GetScreenSize()
##    print GetAreaID()
##    print GetAreaSize()
##    print GetKeyQualifiers() 
##    print GetScreenInfo() 
##    for area in GetScreenInfo():
##        print area, area['type']
##        if area['type'] == 1:
##            print "Add event"
##            #for evnt in range(1,30000):
##            #    print "Event:",evnt
##            #    QAdd( area['id'],evnt,1)
##            print Qual
##            SetKeyQualifiers(Qual['LSHIFT'])
##            #QAdd(area['id'],Draw.LEFTSHIFTKEY,1,1)
##            QAdd( area['id'],Draw.F4KEY,1)
##            #QAdd( area['id'],Draw.GKEY,0)
##    #print Draw
##    #dir(Draw)
##    return

# Udalosti od GUI
def event(event,val):
    #print event, val    
    Draw.Redraw(1)

# Registrace GUI v systemu
Draw.Register( drawGUI, event, button_evt)