import numpy
from scipy import *

import Blender
from Blender import *

from rahat.calibim import *

# Class Calibrated Image
class Silhouette:
    def __init__(self,CalibratedImage):
        self.cim = CalibratedImage 
        self.findBoundaries()
    def findBoundaries(self):
        image = self.cim.im
        minXY = image.getMinXY()
        maxXY = image.getMaxXY()
        self.ref = image.getPixelI(minXY[0],minXY[1])
        self.left=maxXY[0]
        self.right=minXY[0]
        self.top=minXY[1]
        self.bottom=maxXY[1]
        self.silmap = zeros(image.getSize())
        for x in range(minXY[0],maxXY[0]):
            for y in range(minXY[1],maxXY[1]):
                if self.ref != image.getPixelI(x,y):
                    if self.top < y :
                        self.top = y
                    if self.bottom > y:
                        self.bottom = y
                    if self.left > x:
                        self.left = x
                    if self.right < x:
                        self.right = x
                    self.silmap[x,y]=1
    def drawPolyToMap(self,coords,color):
        # First found Bounding Box
        left=coords[0][0]
        right=coords[0][0]
        top=coords[0][1]
        bottom=coords[0][1]        
        for c in coords:
            # note that C is a Matrix Object
            # Also draw a point to the Map
            self.silmap[int(c[0]),int(c[1])]=2
            if c[0] < left: left=c[0]
            if c[0] > right: right=c[0]
            if c[1] < bottom: bottom=c[1]
            if c[1] > top: top=c[1]
        # transformation from Matrix object
        top=top[0,0]
        bottom=bottom[0,0]
        left=left[0,0]
        right=right[0,0]
        # SetupEdges
        # find descending order in y axis of points
        V1 = coords[0]
        V2 = coords[1]
        V3 = coords[2]
        if V1[1] < V3[1]:
            tmp=V3
            V3=V1
            V1=tmp
        if V1[1] < V2[1]:
            tmp=V2
            V2=V1
            V2=tmp
        if V3[1] > V2[1]:
            tmp=V2
            V2=V3
            V2=tmp           
        # now we compute the constants
        # edges are: V1-V2, V2-V3, V3-V1
        c0 = V1[0]*V2[1]-V2[0]*V1[1]
        b0 = V2[0]-V1[0]
        a0 = V1[1]-V2[1]
        c1 = V2[0]*V3[1]-V3[0]*V2[1]
        b1 = V3[0]-V2[0]
        a1 = V2[1]-V3[1]
        c2 = V3[0]*V1[1]-V1[0]*V3[1]
        b2 = V1[0]-V3[0]
        a2 = V3[1]-V1[1]
        # check if these equations are positive for corners
        # otherwise switch
        if (a0*V3[0] + b0*V3[1] + c0) < 0:
            a0=-a0
            b0=-b0
            c0=-c0
        if (a1*V1[0] + b1*V1[1] + c1) < 0:
            a1=-a1
            b1=-b1
            c1=-c1
        if (a2*V2[0] + b2*V2[1] + c2) < 0:
            a2=-a2
            b2=-b2
            c2=-c2
        # Proceed the drawing        
        for y in range(int(bottom),int(top)):
            for x in range(int(left),int(right)):                
                e0 = a0*float(x) + b0*float(y) +c0
                e1 = a1*float(x) + b1*float(y) +c1
                e2 = a2*float(x) + b2*float(y) +c2
                if (e0 >= 0) and (e1 >= 0) and (e2 >= 0):                    
                    self.silmap[x,y]=2
                    self.silmap[x+1,y]=2
                    self.silmap[x+1,y+1]=2
                    self.silmap[x,y+1]=2
                    
    def showMap(self):
        image = self.cim.im
        im = Image.New("silhouette",image.getSize()[0],image.getSize()[1],8)
        for x in range(0,image.getSize()[0]):
            for y in range(0,image.getSize()[1]):
                    im.setPixelI(x,y,(self.silmap[x,y]*127,0,0,0))
                    
        
    
